%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ! This MATLAB script requires the Fuzzy Logic Toolbox ! %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% define variables:
T = (0:0.1:40)';
P = (0:0.1:100)';

% define membership functions:
hot = trapmf(T, [25 35 40 40]);
moderatelyhot = trimf(T, [20 25 30]);
crowdedness = trapmf(P, [25 50 100 100]);

% plot membership functions:
figure(1);
plot(T, hot, '-b', T, moderatelyhot, '-r', 'LineWidth', 2);
legend('hot', 'moderately hot', 'Location','NorthWest');
xlabel('temperature T');

figure(2);
plot(P, crowdedness, '--g', 'LineWidth', 2);
legend('crowdedness', 'Location','NorthWest');
xlabel('percentage of crowdedness C');



% define and plot membership function of the phrase
% 'the temperature is hot OR the temperature is moderately hot'
or = max(hot, moderatelyhot);
figure(3);
plot(T, or, '-r', 'LineWidth', 2);
legend('hot OR moderately hot', 'Location','NorthWest');
xlabel('temperature T');

% evaluate the membership function at T=28C:
actualTempIdx = find(T==28);
eval = or(actualTempIdx);
% add vertical line at T=28C to last plot:
hold on;
grid on;
plot([28,28], [0,1], '--k', 'LineWidth', 2); 
hold off;


% apply Mamdani's and Larsen's rules:
crowdednessAfterMamdani = applyMamdaniRule(eval, crowdedness);
crowdednessAfterLarsen = applyLarsenRule(eval, crowdedness);

% plot resulting membership functions:
figure(4);
plot(P, crowdednessAfterMamdani, '-g', P, crowdednessAfterLarsen, '-m', 'LineWidth', 2);
legend('crowdedness wrt Mamdani', 'crowdedness wrt Larsen', 'Location','NorthWest');
xlabel('percentage of crowdedness C');

% use defuzzify method of Matlab's Fuzzy Logic Toolbox to get a crisp
% answer:
mamdanisAnswer = defuzz(P, crowdednessAfterMamdani, 'centroid');
larsensAnswer =  defuzz(P, crowdednessAfterLarsen, 'centroid');
hold all;
grid on;
plot([mamdanisAnswer,mamdanisAnswer], [0,1], '--g', 'LineWidth', 2); 
plot([larsensAnswer,larsensAnswer], [0,1], '--m', 'LineWidth', 2); 
hold off;
legend('crowdedness wrt Mamdani', 'crowdedness wrt Larsen', 'centroid wrt Mamdani', 'centroid wrt Larsen', 'Location','NorthWest');